package fullstack

import (
	"testing"
	"time"

	"github.com/getsentry/sentry-go"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"xbus.io/go-xbus/v3/api"
)

type TestTransport struct {
	t      *testing.T
	events chan *sentry.Event
}

func NewTestTransport(t *testing.T) *TestTransport {
	t.Helper()

	return &TestTransport{
		t:      t,
		events: make(chan *sentry.Event, 5),
	}
}

func (tt *TestTransport) Close() {
	close(tt.events)
}

func (tt *TestTransport) Flush(timeout time.Duration) bool {
	return true
}

func (tt *TestTransport) Configure(options sentry.ClientOptions) {}

// Send stores the event for later inspection
func (tt *TestTransport) SendEvent(event *sentry.Event) {
	tt.events <- event
}

func TestSentry(t *testing.T) {
	t.Run("CaptureProcessERROR", func(t *testing.T) {
		transport := NewTestTransport(t)
		defer transport.Close()

		sentryClient, err := sentry.NewClient(sentry.ClientOptions{
			Transport: transport,
		})
		require.NoError(t, err)
		t.Log(sentryClient)
		hub := sentry.NewHub(sentryClient, sentry.NewScope())

		ft := InitFullstack(t, false, Sentry(hub, transport))
		if ft == nil {
			return
		}

		if !ft.ServerStart() {
			t.Fatal("Server did not start")
		}
		defer ft.Shutdown()

		ft.CreateRole(api.Actor_EMITTER, "anyemitter")
		_, emitter := ft.CreateEmitter("bob", []string{"anyemitter"})

		_, err = emitter.SendItems(nil, api.DefaultOutput, "evt1", []api.Item{[]byte("item1")})

		require.Equal(t, "No matching pipeline", err.Error())

		select {
		case event := <-transport.events:
			assert.Regexp(t, "\\[process\\]\\[from bob\\] No matching pipeline", event.Message)
		case <-time.After(2 * time.Second):
			t.Error("Timeout: no raven packet sent")
		}
	})
}
