// Copyright 2016 Orus.io. All rights reserved.

package control

import (
	"context"
	"sync"
	"sync/atomic"

	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/api/control"
)

type envelopeServer struct {
	*Service
}

func (s envelopeServer) Purge(
	ctx context.Context,
	send func(control.TaskProgress),
) (err error) {
	defer func() {
		if r := recover(); r != ctx.Err() { //nolint:errorlint
			panic(r)
		}
	}()
	const maxSize = 65000
	var count, total uint32
	s.o.EnvelopeStorage.Purge(func(ids []api.UUID) []api.UUID {
		select {
		case <-ctx.Done():
			panic(ctx.Err())
		default:
		}
		var (
			jobs       = make(chan []api.UUID)
			results    = make(chan []api.UUID)
			batchCount = len(ids) / maxSize
			jobCount   = batchCount
			wg         sync.WaitGroup
		)
		defer close(results)
		defer wg.Wait()
		if len(ids)%maxSize != 0 {
			batchCount++
		}
		if jobCount == 0 {
			jobCount = 1
		}
		if jobCount > 5 {
			jobCount = 5
		}

		for i := 0; i != jobCount; i++ {
			wg.Add(1)
			go func() {
				defer wg.Done()
				for {
					select {
					case ids, ok := <-jobs:
						if !ok {
							return
						}
						var result []api.UUID
						envProcess := s.o.ProcessStorage.FindProcessForEnvelope(ids)
						for _, id := range ids {
							if _, ok := envProcess[id]; !ok {
								result = append(result, id)
							}
						}
						atomic.AddUint32(&total, uint32(len(ids)))
						select {
						case results <- result:
						case <-ctx.Done():
						}
					case <-ctx.Done():
						return
					}
				}
			}()
		}

		wg.Add(1)
		go func() {
			defer wg.Done()
			defer close(jobs)
			for len(ids) != 0 {
				var batch = ids
				if len(batch) > maxSize {
					batch = ids[:maxSize]
				}
				ids = ids[len(batch):]
				select {
				case <-ctx.Done():
					return
				case jobs <- batch:
				}
			}
		}()
		var result []api.UUID
		for i := 0; i != batchCount; i++ {
			select {
			case resIDs := <-results:
				atomic.AddUint32(&count, uint32(len(resIDs)))
				result = append(result, resIDs...)
				send(control.TaskProgress{Progression: atomic.LoadUint32(&count), MaxProgression: atomic.LoadUint32(&total)})
			case <-ctx.Done():
				panic(ctx.Err())
			}
		}
		return result
	})
	return ctx.Err()
}

func (s envelopeServer) GetStorageStat(ctx context.Context) (control.StorageStat, error) {
	entries, err := s.o.EnvelopeStorage.Stat()
	if err != nil {
		return control.StorageStat{}, err
	}
	return control.StorageStat{Entries: entries}, nil
}
