package cache

import "github.com/prometheus/client_golang/prometheus"

var (
	hitmissCounter = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "xbus_storage_cache_hitmiss",
			Help: "Storage cache hit/miss counter",
		},
		[]string{"storage", "name", "operation", "status"},
	)

	sizeInBytesGauge = prometheus.NewGaugeVec(
		prometheus.GaugeOpts{
			Name: "xbus_storage_cache_size_in_bytes",
			Help: "Storage cache size in bytes",
		},
		[]string{"storage", "name"},
	)

	sizeInBytesMaxGauge = prometheus.NewGaugeVec(
		prometheus.GaugeOpts{
			Name: "xbus_storage_cache_size_in_bytes_max",
			Help: "Storage cache maximum size in bytes",
		},
		[]string{"storage", "name"},
	)

	sizeGauge = prometheus.NewGaugeVec(
		prometheus.GaugeOpts{
			Name: "xbus_storage_cache_size",
			Help: "Storage cache: number of elements in cache",
		},
		[]string{"storage", "name"},
	)
)

func init() {
	prometheus.MustRegister(
		hitmissCounter,
		sizeGauge,
		sizeInBytesGauge,
		sizeInBytesMaxGauge,
	)
}
